@@ -148,19 +148,91 @@ class NeuralNetwork:
             sum += math.pow(err, 2)
         return sum / len(errors)
 
-    def to_function(self, fnname='nn_run'):
-        fn = """
-def {fnname}(i):
-	_ = {weights}
-	o = [i]
-	for l in range(1, {layerCount}):
-		o.append([])
-		for n in range({sizes}[l]):
-			sum = {biases}[l][n]
-			for k in range(len(_[l][n])):
-				sum += _[l][n][k] * i[k]
-			o[-1].append(1 / (1 + math.exp(-sum)))
-		i = o[l]
-	return o[-1]
-""".format(fnname=fnname, layerCount=self.outputLayer + 1, weights=repr(self.weights), sizes=repr(self.sizes), biases=repr(self.biases)).replace('\t', '    ')
-        return fn
+    def _indent(self, txt, chars):
+        result = ''
+        d = ' ' * chars
+        for line in txt.split('\n'):
+            result += d + line + '\n'
+        return result
+
+    def to_function(self, fnname='nn_run', indent=0):
+        fn = 'def {fnname}(i):\n'.format(fnname=fnname)
+        for l in range(1, self.outputLayer + 1):
+            if l < self.outputLayer:
+                fn += '    o = [\n'
+            else:
+                fn += '    return [\n'
+            size = self.sizes[l]
+            for n in range(size):
+                term = str(-self.biases[l][n])
+                length = len(self.weights[l][n])
+                for k in range(length):
+                    w = self.weights[l][n][k]
+                    term = term + ('-' if w > 0 else '+') + \
+                        str(abs(w)) + '*i[' + str(k) + ']'
+                fn += '        1/(1+math.exp(' + term + '))' + \
+                    (',' if n != size - 1 else '') + '\n'
+            fn += '    ]\n'
+            if l != self.outputLayer:
+                fn += '    i = o\n'
+        return self._indent(fn, indent)
+
+    def to_java_method(self, fnname='nn_run', static=False, scope='protected', indent=4):
+        fn = scope + (' static ' if static else ' ') + \
+            'double[] {fnname}(double[] i)'.format(fnname=fnname) + '{\n'
+        fn += '    double[] o;\n'
+        for l in range(1, self.outputLayer + 1):
+            if l < self.outputLayer:
+                fn += '    o = new double[]{\n'
+            else:
+                fn += '    return new double[]{\n'
+            size = self.sizes[l]
+            for n in range(size):
+                term = str(-self.biases[l][n])
+                length = len(self.weights[l][n])
+                for k in range(length):
+                    w = self.weights[l][n][k]
+                    term = term + ('-' if w > 0 else '+') + \
+                        str(abs(w)) + '*i[' + str(k) + ']'
+                fn += '        1/(1+Math.exp(' + term + '))' + \
+                    (',' if n != size - 1 else '') + '\n'
+            fn += '    };\n'
+            if l != self.outputLayer:
+                fn += '    i = o;\n'
+        fn += '}'
+        return self._indent(fn, indent)
+
+    def to_c_function(self, fnname='nn_run', indent=0):
+        fn = 'void {fnname}(double *i, double *o)'.format(fnname=fnname) + \
+            '{\n'
+        terms = {}
+        oterms = {}
+        lterms = []
+        for k in range(self.sizes[0]):
+            lterms.append('o0_' + str(k))
+            terms[lterms[-1]] = 'i[' + str(k) + ']'
+        for l in range(1, self.outputLayer + 1):
+            size = self.sizes[l]
+            for n in range(size):
+                term = str(-self.biases[l][n])
+                length = len(self.weights[l][n])
+                for k in range(length):
+                    w = self.weights[l][n][k]
+                    term = term + ('-' if w > 0 else '+') + \
+                        str(abs(w)) + '*o' + str(l - 1) + '_' + str(k)
+                v = '(1/(1+exp(' + term + ')))'
+                for k in lterms:
+                    v = v.replace(k, terms[k])
+                lterms.append('o' + str(l) + '_' + str(n))
+                terms[lterms[-1]] = v
+                if l == self.outputLayer:
+                    oterms['o' + str(l) + '_' + str(n)] = 'o[' + str(n) + ']'
+        # for t in lterms:
+        #    if not t in oterms:
+        #        fn += '    double '+t+'='+terms[t]+';\n'
+        #    else:
+        #        fn += '    ' + oterms[t]+' = ' + terms[t]+';\n'
+        for k, v in oterms.items():
+            fn += '    ' + v + ' = ' + terms[k] + ';\n'
+        fn += '}\n'
+        return self._indent(fn, indent)
